function [phi,Rg] = fdm_solver(data,Kg,Rg)
%
%    [phi,Rg] = fdm_solver(data,Kg,Rg)
%
%
%   Rutina SOLVER de resolucion del sistema de ecuaciones
%
%   Type_solver = 0 ---> metodo directo con un parametro de
%                        parametro de relajacion "relax"
%                        grado de implicitud (1/2 Cranck Nicholson) "alpha" (use >0)
%               = 1 ---> Line search
%               = 2 ---> Explicito - Forward Euler
%               = 3 ---> GMRES
%               = 4 ---> QMR (quasi minimum residual)
%

%disp(' at SOLVER ')
%keyboard;pause

relax = data.param.relax;

nodes_free = (1:size(data.xnod,1))';
nodes_free(data.bc.Dirichlet.nodes) = [];

phi = data.state;
ndf = data.param.ndf;
scaling = data.param.scaling;

Rg = Rg(nodes_free);
Kg = Kg(nodes_free,nodes_free);

if scaling,
    scalev = sum(abs(Kg'))';
    Rg   = Rg ./ scalev;
    Kg   = diag(scalev)\Kg;
end

dphi = -Kg\Rg;
phiv = m2v(phi);
phiv(nodes_free) = phiv(nodes_free)+ relax*dphi;
phi  = v2m(phiv,ndf);
